!
! Copyright (c) 2000-2013 Yoshihide Yoshimoto
! 
module util
contains
  real(kind=8) function cellvol(aa)
    !
    !       calculates the cell volume of aa
    !
    implicit none
    real(kind=8),intent(in):: aa(3,3)
    !
    real(kind=8) v
    !
    v=aa(1,1)*(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3)) &
         +aa(2,1)*(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3)) &
         +aa(3,1)*(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))
    cellvol=abs(v)
    return
  end function cellvol
  !
  subroutine outer_product(a,b,c)
    !
    !       calculates the outer product c = a \cross b
    !
    implicit none
    real(kind=8),intent(in):: a(3),b(3)
    real(kind=8),intent(out):: c(3)
    !        
    c(1) = a(2)*b(3)-a(3)*b(2)
    c(2) = a(3)*b(1)-a(1)*b(3)
    c(3) = a(1)*b(2)-a(2)*b(1)
    return
  end subroutine outer_product
  !
  integer function suplatp(aa,r)
    implicit none
    real(kind=8),intent(in)::aa(3,3),r
    !
    real(kind=8),parameter:: pi = 3.1415926535897932385d0
    real(kind=8):: x(3),d
    !
    ! calculate cell diameter d.
    !
    x(:) = aa(:,1) + aa(:,2) + aa(:,3)
    d = sqrt(dot_product(x,x))
    !
    x(:) = aa(:,1) + aa(:,2) - aa(:,3)
    d = max(d, sqrt(dot_product(x,x)))
    !
    x(:) = aa(:,1) - aa(:,2) + aa(:,3)
    d = max(d, sqrt(dot_product(x,x)))
    !
    x(:) = aa(:,1) - aa(:,2) - aa(:,3)
    d = max(d, sqrt(dot_product(x,x)))
    !
    suplatp = int((4.0d0*pi/3.0d0)*(r+d)**3/cellvol(aa))
    return
  end function suplatp
  !
  subroutine LatticeInterval(aa,h)
    implicit none
    real(kind=8),intent(in):: aa(3,3)
    real(kind=8),intent(out):: h(3)
    !
    real(kind=8):: hh(3,3),htmp
    !
    ! hh(*,1) = aa(*,2) \times aa(*,3)
    ! hh(*,2) = aa(*,3) \times aa(*,1)
    ! hh(*,3) = aa(*,1) \times aa(*,2)
    !
    call outer_product(aa(:,2),aa(:,3),hh(:,1))
    call outer_product(aa(:,3),aa(:,1),hh(:,2))
    call outer_product(aa(:,1),aa(:,2),hh(:,3))
    !
    ! normalize hh(*,1),hh(*,2),hh(*,3)
    !
    htmp=sqrt(dot_product(hh(:,1),hh(:,1)))
    hh(:,1)=hh(:,1)/htmp
    htmp=sqrt(dot_product(hh(:,2),hh(:,2)))
    hh(:,2)=hh(:,2)/htmp
    htmp=sqrt(dot_product(hh(:,3),hh(:,3)))
    hh(:,3)=hh(:,3)/htmp
    !
    ! h(1): interval of the plane (1,0,0) etc.
    !
    h(1)=abs(dot_product(aa(:,1),hh(:,1)))
    h(2)=abs(dot_product(aa(:,2),hh(:,2)))
    h(3)=abs(dot_product(aa(:,3),hh(:,3)))
    return
  end subroutine LatticeInterval
  !
  subroutine fctck(n,m2,m3,m5,info)
    implicit none
    integer,intent(in):: n
    integer,intent(out):: m2, m3, m5, info
    !
    integer:: i
    !
    i=n
    m2 = 0
    m3 = 0
    m5 = 0
    info = 0
    !
    do while (i .ne. 1)
       if (mod(i,2) .eq. 0) then
          m2 = m2 + 1
          i = i / 2
       else if (mod(i,3) .eq. 0) then
          m3 = m3 + 1
          i = i / 3
       else if (mod(i,5) .eq. 0) then
          m5 = m5 + 1
          i = i / 5
       else
          info = 1
          exit
       end if
    end do
    return
  end subroutine fctck
!
  integer function algn235(inr,gctr)
    implicit none
    integer,intent(in):: inr,gctr
!
    integer:: nr,m2,m3,m5,info
    integer:: t2, t3, t5
!
    call fctck(gctr,t2,t3,t5,info)
    if (info.eq.1) then
       write(6,*) 'algn235: gctr is not a multiple of 2,3,5'
       stop
    end if
    nr = inr
    call fctck(nr,m2,m3,m5,info)
    do while ( (info .eq. 1) .or. (m2.lt.t2) &
         .or. (m3.lt.t3) .or. (m5.lt.t5) )
       nr = nr + 1
       call fctck(nr,m2,m3,m5,info)
    end do
    algn235 = nr
    return
  end function algn235
!
  subroutine cell2cart(aa,n,vcell,vcart)
    implicit none
    integer:: n
    real(kind=8):: aa(3,3),vcell(3,n),vcart(3,n)
    integer:: i,j,k

    vcart = 0.0d0
    do i=1,n
       do j=1,3
          do k=1,3
             vcart(k,i) = vcart(k,i) + aa(k,j)*vcell(j,i)
          end do
       end do
    end do
  end subroutine cell2cart

  subroutine cart2cell(aa,n,vcart,vcell)
    implicit none
    integer:: n
    real(kind=8):: aa(3,3),vcell(3,n),vcart(3,n)
    integer:: i,j,k
    real(kind=8):: aai(3,3),w

    w=aa(1,1)*(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3)) &
         +aa(2,1)*(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3)) &
         +aa(3,1)*(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))

    aai(1,1)=(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3))/w
    aai(1,2)=(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3))/w
    aai(1,3)=(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))/w
    aai(2,1)=(aa(2,3)*aa(3,1)-aa(3,3)*aa(2,1))/w
    aai(2,2)=(aa(3,3)*aa(1,1)-aa(1,3)*aa(3,1))/w
    aai(2,3)=(aa(1,3)*aa(2,1)-aa(2,3)*aa(1,1))/w
    aai(3,1)=(aa(2,1)*aa(3,2)-aa(3,1)*aa(2,2))/w
    aai(3,2)=(aa(3,1)*aa(1,2)-aa(1,1)*aa(3,2))/w
    aai(3,3)=(aa(1,1)*aa(2,2)-aa(2,1)*aa(1,2))/w

    vcell = 0.0d0
    do i=1,n
       do j=1,3
          do k=1,3
             vcell(k,i) = vcell(k,i) + aai(k,j)*vcart(j,i)
          end do
       end do
    end do
  end subroutine cart2cell

  subroutine cart2cellr(aa,n,vcart,vcellr)
    implicit none
    integer:: n
    real(kind=8):: aa(3,3),vcellr(3,n),vcart(3,n)
    integer:: i,j,k
    real(kind=8):: aai(3,3),w

    w=aa(1,1)*(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3)) &
         +aa(2,1)*(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3)) &
         +aa(3,1)*(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))

    aai(1,1)=(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3))/w
    aai(1,2)=(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3))/w
    aai(1,3)=(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))/w
    aai(2,1)=(aa(2,3)*aa(3,1)-aa(3,3)*aa(2,1))/w
    aai(2,2)=(aa(3,3)*aa(1,1)-aa(1,3)*aa(3,1))/w
    aai(2,3)=(aa(1,3)*aa(2,1)-aa(2,3)*aa(1,1))/w
    aai(3,1)=(aa(2,1)*aa(3,2)-aa(3,1)*aa(2,2))/w
    aai(3,2)=(aa(3,1)*aa(1,2)-aa(1,1)*aa(3,2))/w
    aai(3,3)=(aa(1,1)*aa(2,2)-aa(2,1)*aa(1,2))/w

    vcellr = 0.0d0
    do i=1,n
       do j=1,3
          do k=1,3
             vcellr(k,i) = vcellr(k,i) + aai(k,j)*vcart(j,i)
          end do
       end do
       do k=1,3
          vcellr(k,i) = modulo(vcellr(k,i),1.0d0)
       end do
    end do
  end subroutine cart2cellr

  !
  ! transform contravariant vectors
  ! from cartesian coordinate to cell coordinate
  !
  subroutine acart2cell(aa,n,vcart,vcell)
    implicit none
    integer:: n
    real(kind=8):: aa(3,3),vcart(3,n),vcell(3,n)
    integer:: i,j,k

    vcell = 0.0d0
    do i=1,n
       do j=1,3
          do k=1,3
             vcell(k,i) = vcell(k,i) + aa(j,k)*vcart(j,i)
          end do
       end do
    end do
  end subroutine acart2cell

  subroutine recp_uvec(aa,bb)
    implicit none
    real(kind=8),intent(in):: aa(3,3)
    real(kind=8),intent(out):: bb(3,3)
    real(kind=8),parameter:: pi = 3.1415926535897932385d0

    real(kind=8):: v

    v=aa(1,1)*(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3)) &
         +aa(2,1)*(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3)) &
         +aa(3,1)*(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))
    v=v/(2.0d0*pi)

    bb(1,1)=(aa(2,2)*aa(3,3)-aa(3,2)*aa(2,3))/v
    bb(2,1)=(aa(3,2)*aa(1,3)-aa(1,2)*aa(3,3))/v
    bb(3,1)=(aa(1,2)*aa(2,3)-aa(2,2)*aa(1,3))/v
    bb(1,2)=(aa(2,3)*aa(3,1)-aa(3,3)*aa(2,1))/v
    bb(2,2)=(aa(3,3)*aa(1,1)-aa(1,3)*aa(3,1))/v
    bb(3,2)=(aa(1,3)*aa(2,1)-aa(2,3)*aa(1,1))/v
    bb(1,3)=(aa(2,1)*aa(3,2)-aa(3,1)*aa(2,2))/v
    bb(2,3)=(aa(3,1)*aa(1,2)-aa(1,1)*aa(3,2))/v
    bb(3,3)=(aa(1,1)*aa(2,2)-aa(2,1)*aa(1,2))/v
    return
  end subroutine recp_uvec

  subroutine hsort(n, d, idx)
    implicit none
    integer,intent(in):: n
    real(kind=8):: d(n)
    integer:: idx(n)

    integer:: i, j, ip, ic, it
    real(kind=8):: t

    ! initialize idx
    do i=1,n
       idx(i) = i
    end do

    ! create the heap
    do i=2,n
       j = i
       ip = j/2
       do while ( d(j) .gt. d(ip) )
          t = d(ip)
          it = idx(ip)
          d(ip) = d(j)
          idx(ip) = idx(j)
          d(j) = t
          idx(j) = it
          j = ip
          ip = j/2
          if ( ip .eq. 0 ) exit
       end do
    end do

    ! sort it
    do i=n,1,-1
       t = d(i)
       it = idx(i)
       d(i) = d(1)
       idx(i) = idx(1)
       d(1) = t
       idx(1) = it
       j = 1
       ic = 2*j
       do while ( ic .lt. i )
          if ( ic + 1 .lt. i ) then
             if ( d(ic+1) .gt. d(ic) ) ic = ic + 1
          end if
          if ( d(ic) .gt. d(j) ) then
             t = d(ic)
             it = idx(ic)
             d(ic) = d(j)
             idx(ic) = idx(j)
             d(j) = t
             idx(j) = it
             j = ic
          else
             exit
          end if
          ic = 2*j
       end do
    end do
    return
  end subroutine hsort

end module util
